package eu.clarin.cmdi.mdservice.model;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;

import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import net.sf.json.JSON;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import net.sf.json.JSONSerializer;
import net.sf.json.xml.XMLSerializer;
import eu.clarin.cmdi.mdservice.internal.MDTransformer;
import eu.clarin.cmdi.mdservice.internal.Utils;
import eu.clarin.cmdi.mdservice.action.FCSAction;
import eu.clarin.cmdi.mdservice.action.WorkspaceAction;
import eu.clarin.cmdi.mdservice.internal.NoStylesheetException;

/**
 
 */



public class FCSRepositories{	

	private static FCSRepositories singleton;
	private static Document fcsrepositories_document;
	
	public static HashMap<String,String> fcs_mapping;
	
	public FCSRepositories() {
		if (fcsrepositories_document == null){
			Init();
		}
	}
	
	public static FCSRepositories getFCSRepositories() {
		if (singleton == null) {
			singleton = new FCSRepositories();
		} 
		return singleton;
	}
	
	public void Init(){
		NodeList ilist, nlist;
		String type = null, name = null, uri = null;
		NodeList fcsnodes, fcsrepositories_nodelist;
		Document fcsdoc;
		Node fcsnode;
		
		Repositories rep = Repositories.getRepositories();
		fcsrepositories_nodelist = (NodeList ) rep.getRepositoriesNodeList();
		nlist = fcsrepositories_nodelist.item(0).getChildNodes();
		
		fcs_mapping = new HashMap<String,String>();
		fcsrepositories_document  = Utils.createDocument("Repositories");
		
		Element keycontainer = fcsrepositories_document.getDocumentElement(); 
        //keycontainer.setAttributeNS(null, "Version", "1.0"); 
		//xmlns:sru="http://www.loc.gov/zing/srw/
        keycontainer.setAttributeNS("http://www.w3.org/2000/xmlns/",
                "xmlns:srw",
                "http://www.loc.gov/zing/srw/");
        keycontainer.setAttributeNS("http://www.w3.org/2000/xmlns/",
                "xmlns:sru",
                "http://www.loc.gov/zing/srw/");

		nlist = fcsrepositories_nodelist.item(0).getChildNodes();
		for(int i=0;i<nlist.getLength();i++){
			if (nlist.item(i).getNodeName().equals("item")){
				fcsnode = fcsrepositories_document.adoptNode(nlist.item(i).cloneNode(true));
				fcsrepositories_document.getChildNodes().item(0).appendChild(fcsnode);
				
				ilist  = fcsnode.getChildNodes();
				for(int j=0;j<ilist.getLength();j++){
					if (ilist.item(j).getNodeName().equals("type"))
						type = ilist.item(j).getFirstChild().getNodeValue();
					if (ilist.item(j).getNodeName().equals("name"))
						name = ilist.item(j).getFirstChild().getNodeValue();
					if (ilist.item(j).getNodeName().equals("uri"))
						uri = ilist.item(j).getFirstChild().getNodeValue();
				}
				if (type.equals("fcs.resource")){
					fcsdoc = FCSRepositories.getFCSData(name);
					addFCS2Repositories(fcsnode, fcsdoc, name);
				}
			}
		}
	}
	
	public String getFCSMapping(String context){
		return this.fcs_mapping.get(context);
	}
	
	public void addFCS2Repositories(Node node, Document fcs_doc, String name){
		Node extraTermNode, extraTermNodeAdopted, eNode;
		String ename = null;
		NodeList termsnodelist, termlist;
		
		NodeList elist = fcs_doc.getElementsByTagName("sru:extraTermData");
		for(int i=0;i<elist.getLength();i++){
			extraTermNode = elist.item(i);
			eNode = extraTermNode;
			while (!eNode.getNodeName().equals("sru:value")){
				eNode = eNode.getPreviousSibling();
			}
			if (eNode.getTextContent().equals(name)){
				extraTermNodeAdopted = fcsrepositories_document.adoptNode(extraTermNode.cloneNode(true));
				node.appendChild(extraTermNodeAdopted);
				termsnodelist = extraTermNodeAdopted.getChildNodes().item(1).getChildNodes();
				for(int k=0;k<termsnodelist.getLength();k++){
					termlist = termsnodelist.item(k).getChildNodes();
					for(int l=0;l<termlist.getLength();l++){
						if (termlist.item(l).getNodeName().equals("sru:value")){
							ename = termlist.item(l).getTextContent();
							this.fcs_mapping.put(ename, name);
						}
					}
				}
			}
		}
	}
	
	public static Document getFCSData(String name){
		
		Document doc;
		
		FCSAction fcsa = new FCSAction();
		fcsa.setActionkey("fcs");
		fcsa.setParam("operation", "scan");
		fcsa.setParam("scanClause", "fcs.resource");
		fcsa.setParam("x-context", name);
		
		try {
			fcsa.execute();
			
			DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
         	DocumentBuilder docBuilder;
			try {
				docBuilder = docFactory.newDocumentBuilder();		    
				try {
						doc = docBuilder.parse(fcsa.getResultStream());
						return doc;
				} catch (SAXException e) {
						// TODO Auto-generated catch block
						
						e.printStackTrace();
				}
			} catch (ParserConfigurationException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
		
	}
	
	public InputStream getXMLStream() throws TransformerConfigurationException, TransformerException, TransformerFactoryConfigurationError{
		
		InputStream srustream = null;
		InputStream xmlstream = Utils.document2Stream(fcsrepositories_document.getFirstChild());
		// convert to sru
		MDTransformer transformer = new MDTransformer();
		transformer.configure(Utils.getAppConfig("mdservice"), this.getClass().getClassLoader());
		//transformer.setSrcFile(getTargetRequest());
		transformer.setParams(MDTransformer.createParamsMap("xml2sruscan"));
		try {
			srustream = transformer.transformXML(xmlstream);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoStylesheetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return srustream;
		
	}


	
}


